package com.usky.iot.controller.web;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.bean.CommonPage;
import com.usky.iot.domain.BaseFacilityDevice;
import com.usky.iot.domain.DmpDeviceInfo;
import com.usky.iot.service.DmpDeviceInfoService;
import com.usky.iot.service.vo.BaseFacilityDeviceVO;
import com.usky.iot.service.vo.DmpDeviceInfoRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.List;

/**
 * 设备信息表
 *
 * @author ya
 * @since 2022-10-08
 */
@RestController
@RequestMapping("/dmpDeviceInfo")
public class DmpDeviceInfoController {

    @Autowired
    private DmpDeviceInfoService dmpDeviceInfoService;

    /**
     * 新增
     * @param dmpDeviceInfo
     * @return
     */
    @PostMapping
    public ApiResult<Void> add(@RequestBody DmpDeviceInfo dmpDeviceInfo){
        dmpDeviceInfoService.add(dmpDeviceInfo);
        return ApiResult.success();
    }

    /**
     * 修改
     * @param dmpDeviceInfo
     * @return
     */
    @PutMapping
    public ApiResult<Void> edit(@RequestBody DmpDeviceInfo dmpDeviceInfo){
        dmpDeviceInfoService.update(dmpDeviceInfo);
        return ApiResult.success();
    }


    /**
     * 分页
     * @param dmpDeviceInfoRequest
     * @return
     */
    @PostMapping("/page")
    public ApiResult<CommonPage<DmpDeviceInfo>> page(@RequestBody DmpDeviceInfoRequest dmpDeviceInfoRequest)
    {
        return ApiResult.success(dmpDeviceInfoService.page(dmpDeviceInfoRequest));
    }

    /**
     * 删除记录
     */
    @DeleteMapping("/{id}")
    public ApiResult<Void> remove(@PathVariable("id") Integer id)
    {
        dmpDeviceInfoService.remove(id);
        return ApiResult.success();
    }

    /**
     * 批量导入
     * @param multipartFiles
     * @return
     */
    @PostMapping("/addByFile")
    public ApiResult<Void> addByFile(@RequestParam(value = "file") MultipartFile multipartFiles){
        dmpDeviceInfoService.addByFile(multipartFiles);
        return ApiResult.success();
    }

    /**
     * 批量导入-对接客户
     *
     * @param list
     * @return
     */
    @PostMapping("/importDeviceInfo")
    public ApiResult<Void> importDeviceInfo(@RequestBody List<DmpDeviceInfo> list){
        dmpDeviceInfoService.importDeviceInfo(list);
        return ApiResult.success();
    }

    /**
     * 导出
     * @param dmpDeviceInfoRequest
     * @param response
     * @return
     */
    @PostMapping("/export")
    public void export(@RequestBody DmpDeviceInfoRequest dmpDeviceInfoRequest, HttpServletResponse response){
        dmpDeviceInfoService.export(dmpDeviceInfoRequest,response);
    }

    /**
     * 数据中心-设备实时心跳数据接口
     *
     * @param deviceName 设备名称
     * @param installAddress 设备安装位置
     * @param productCode   产品编码
     * @param pageNum  当前页
     * @param pageSize  每页条数
     * @return
     */
    @GetMapping("deviceCurrentDataList")
    public ApiResult<CommonPage<Object>> deviceCurrentDataList(@RequestParam(value = "deviceName",required = false) String deviceName,
                                                               @RequestParam(value = "installAddress",required = false) String installAddress,
                                                               @RequestParam(value = "productCode",required = false) String productCode,
                                                               @RequestParam(value = "pageNum", required = false, defaultValue = "1") Integer pageNum,
                                                               @RequestParam(value = "pageSize", required = false, defaultValue = "10") Integer pageSize){
        return ApiResult.success(dmpDeviceInfoService.deviceCurrentDataList(deviceName,installAddress,productCode,pageNum,pageSize));
    }
}

